(define-module (ordo play)
  #:use-module (oop goops)
  #:use-module (ice-9 match)
  #:use-module (srfi srfi-1) ; list utils
  #:use-module (srfi srfi-9) ; records
  #:use-module (srfi srfi-26) ; cut
  #:use-module (ordo connection)
  #:use-module (ordo context)
  #:use-module (ordo task)
  #:use-module (ordo handler)
  #:export (play run-play))

(define-record-type <play>
  (make-play description connection vars tasks handlers)
  play?
  (connection play-connection)
  (vars play-vars)
  (description play-description)
  (tasks play-tasks)
  (handlers play-handlers))

(define* (play description #:key connection tasks (vars '()) (handlers '()))
  (unless connection (error "connection is required"))
  (unless (is-a? connection <connection>) (error (format #f "invalid connection: ~a" connection)))
  (unless (and tasks (not (null? tasks))) (error "tasks are required"))
  (unless (every task? tasks) (error "invalid tasks"))
  (unless (every (compose handler? cdr) handlers) (error "invalid handlers"))
  (for-each (lambda (task)
              (for-each (lambda (trigger)
                          (unless (assoc-ref handlers trigger)
                            (error (format #f "task \"~a\" references an undefined trigger: ~a"
                                           (task-description task)
                                           trigger))))
                        (task-triggers task)))
            tasks)
  (make-play description connection (fold (match-lambda* (((k . v) accum) (alist-cons k v accum))) '() vars) tasks handlers))

(define (run-play play)
  (format #t "RUNNING PLAY ~a~%" (play-description play))
  (call-with-connection
   (play-connection play)
   (lambda (c)
     (let* ((tmp-dir (car (connection-must c "mktemp" '("--directory"))))
            (ctx (make-context c (play-vars play) tmp-dir)))
       (pk ctx)
       (dynamic-wind
         (const #t)
         (lambda ()
           (for-each (cut run-task ctx <>) (play-tasks play))
           (for-each (match-lambda
                       ((name . handler)
                        (when (context-triggered? ctx name)
                          (run-handler ctx handler))))
                     (play-handlers play)))
         (lambda () (connection-must c "rm" `("-rf" ,tmp-dir))))))))
